/*
 * pdsit.h
 * 
 * Copyright 2015 Fernando Pujaico Rivera <fernando.pujaico.rivera@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 * 
 */

/** \file pdsit.h
 *  \brief Archivo de definición de funciones en teoria de información.
 *
 *  \author Fernando Pujaico Rivera
 */
#ifndef __PDSIT_H__
#define __PDSIT_H__


#ifdef __cplusplus
extern "C" {
#endif 

#include <pds/pdsbvector.h>
#include <pds/pdsvector.h>

#include <stdio.h>

#ifndef TRUE
	#define TRUE 1
#endif

#ifndef FALSE
	#define FALSE 0
#endif

/** \defgroup PdsITGroup funciónes del módulo PdsIT.
 * @{
 */


/** \fn int pds_joint_probability_bsc_model(PdsBVector *OmegaM,const PdsVector *Ps,double p0, double *JointProb)
 *  \brief Encuentra la probabilidad conjunta \f$P(\Omega_M)\f$ de tener en la salida 
 *  de un grupo de canales BSC un conjunto de valores binarios formando un vector 
 *  \f$\Omega_M\f$. 
 *
 *  Conocido un conjunto \f$S=\{U_1, U_2, ..., U_M\}\f$ de fuentes binarias,
 *  estas son construidas pasando la fuente binaria
 *  \f$U_0\f$ con \f$ P(U_0=1)=p_0 \f$, por M canales BSC con probabilidad
 *  de error \f$ P(U_m \neq U_0| U_0)={P_s}_{m}~\forall U_m \in S  \f$.
 *  Si definimos el vector \f$\Omega_M\f$ como:
 *  \f[ \Omega_M \equiv \{u_m: U_m=u_m \forall U_m \in S\} \f]
 *  Entonces este tendrá como probabilidad:
 *  \f{eqnarray*}{
 *  P(\Omega_M)& = & P(U_1=u_1|U_0=0) ... P(U_M=u_M|U_0=0) P(U_0=0) \\
 *  ~          & + & P(U_1=u_1|U_0=1) ... P(U_M=u_M|U_0=1) P(U_0=1)

 *  \f}
 *
 *  \param[in] OmegaM Es un caso de ocurrencia del vector binario.
 *  \param[in] Ps Es el vector de probabilidades de error de las fuentes BSC.
 *  Ps=[Ps0, Ps1, ..., Ps(M-1)]. \f$ {P_s}_{m} \rightarrow Ps(m-1)\f$.
 *  \param[in] p0 Probabilida Pr(U0=1)=p0 de la fuente U0.
 *  \param[out] JointProb probabilidad conjuta del caso de ocurrencia de OmegaM.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: OmegaM==NULL, Ps==NULL.
 *  \ingroup PdsITGroup
 */
int pds_joint_probability_bsc_model(PdsBVector *OmegaM,const PdsVector *Ps,double p0, double *JointProb);

/** \fn int pds_joint_entropy_bsc_model(const PdsVector *Ps,double p0, double *HJoint)
 *  \brief Encuentra la entropia conjunta \f$ H(U_1,...,U_M)\f$ para M fontes generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=p0, atraves de M canales BSC
 *  con probabilidades de error Ps=[Ps0, Ps1, ..., Ps(M-1)].
 *
 *  \f[ H(U_1, U_2, ..., U_M)= \sum_{U_1, U_2, ... U_M} P(U_1, U_2, ..., U_M)~log_2(P(U_1, U_2, ..., U_M)) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)={P_s}_{m} \f]
 *
 *  \param[in] Ps Es el vector de probabilidades de error de las fuentes BSC.
 *  Ps=[Ps0, Ps1, ..., Ps(M-1)]. \f$ {P_s}_{m} \rightarrow Ps(m-1)\f$.
 *  \param[in] p0 Probabilida Pr(U0=1)=p0.
 *  \param[out] HJoint Es el resultado, la entropia conjuta.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: Ps==NULL.
 *  \ingroup PdsITGroup
 */
int pds_joint_entropy_bsc_model(const PdsVector *Ps,double p0, double *HJoint);

/** \fn int pds_probability_u0_omega_bsc_model(PdsBVector *OmegaM,const PdsVector *Ps,PdsBaBit U0,double p0, double *PcU0OmegaM, double *PjU0OmegaM)
 *  \brief Encuentra las probabilidades  condicionada \f$P(u_0 |\Omega_M)\f$ y
 *  conjunta \f$P(u_0 \Omega_M)\f$ .
 *
 *
 *  Conocido un conjunto \f$S=\{U_1, U_2, ..., U_M\}\f$ de fuentes binarias,
 *  estas son construidas pasando la fuente binaria
 *  \f$U_0\f$ con \f$ P(U_0=1)=p_0 \f$, por M canales BSC con probabilidad
 *  de error \f$ P(U_m \neq U_0| U_0)={P_s}_{m}~\forall U_m \in S  \f$.
 *  Asi, definimos el vector \f$\Omega_M\f$ como:
 *  \f[ \Omega_M \equiv \{u_m: U_m=u_m \forall U_m \in S\} \f]
 *  Entonces este tendrá como probabilidad:
 *  \f[
 *  P(u_0\Omega_M) =  P(U_1=u_1|U_0=u_0) ... P(U_M=u_M|U_0=u_0) P(U_0=u_0) 
 *  \f]
 *  \f[
 *  P(u_0|\Omega_M) =  \frac{P(u_0\Omega_M) }{P(\Omega_M)}
 *  \f]
 *  \f{eqnarray*}{
 *  P(\Omega_M)& = & P(U_1=u_1|U_0=0) ... P(U_M=u_M|U_0=0) P(U_0=0) \\
 *  ~          & + & P(U_1=u_1|U_0=1) ... P(U_M=u_M|U_0=1) P(U_0=1)

 *  \f}
 *
 *  \param[in] OmegaM Es un caso de ocurrencia del vector binario.
 *  \param[in] Ps Es el vector de probabilidades de error de las fuentes BSC.
 *  Ps=[Ps0, Ps1, ..., Ps(M-1)]. \f$ {P_s}_{m} \rightarrow Ps(m-1)\f$.
 *  \param[in] u0 Valor de U0 a analiza.
 *  \param[in] p0 Probabilida Pr(U0=1)=p0 de la fuente U0.
 *  \param[out] PcU0OmegaM Retorna la probabilidad condicionada. \f$P(u_0 |\Omega_M)\f$.
 *  \param[out] PjU0OmegaM Retorna la probabilidad conjuta. \f$P(u_0 \Omega_M)\f$.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: OmegaM==NULL, Ps==NULL.
 *  \ingroup PdsITGroup
 */
int pds_probability_u0_omega_bsc_model(PdsBVector *OmegaM,const PdsVector *Ps,PdsBaBit U0,double p0, double *PcU0OmegaM, double *PjU0OmegaM);

/** \fn int pds_entropy_u0_omega_bsc_model(const PdsVector *Ps,double p0, double *HCond)
 *  \brief Encuentra la entropia condicionada. \f$ H(U_0|U_1,...,U_M)\f$.
 *
 *  Dadas M fontes Um , m ={1,2,...,M}, generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=p0, atraves de M canales BSC
 *  con probabilidades de error Ps=[Ps0, Ps1, ..., Ps(M-1)].
 *
 *  \f[ H(U_0|U_1, U_2, ..., U_M)= \sum_{U_0,U_1, U_2, ... U_M} P(U_0,U_1, U_2, ..., U_M)~log_2(P(U_0|U_1, U_2, ..., U_M)) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)={P_s}_{m} \f]
 *
 *  \param[in] Ps Es el vector de probabilidades de error de las fuentes BSC.
 *  Ps=[Ps0, Ps1, ..., Ps(M-1)]. \f$ {P_s}_{m} \rightarrow Ps(m-1)\f$.
 *  \param[in] p0 Probabilida Pr(U0=1)=p0.
 *  \param[out] HCond Entropia condicionada. \f$ H(U_0|U_1,...,U_M)\f$.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: Ps==NULL.
 *  \ingroup PdsITGroup
 */
int pds_entropy_u0_omega_bsc_model(const PdsVector *Ps,double p0, double *HCond);


/** \fn int pds_symetric_entropy_u0_omega_bsc_model(double rho,PdsBaNatural M, double *HCond)
 *  \brief Encuentra la entropia condicionada. \f$ h(U_0| \rho,M)=H(U_0|U_1,...,U_M)\f$.
 *  Este es el caso cuando todas las probabilidades de error son iguales a rho. \f$rho=\rho\f$.
 *
 *  Dadas M fontes Um , m ={1,2,...,M}, generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=0.5, atraves de M canales BSC
 *  con probabilidades de error Ps=[rho, rho, ..., rho].
 *
 *  \f[ h(U_0| \rho,M)= \sum_{k=0}^{M} {M \choose k} \rho^k (1-\rho)^{M-k}~log_2 \left ( 1 + \{ \frac{\rho}{1-\rho} \}^{M-2 k} \right ) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)=\rho \f]
 *
 *  \param[in] rho Es la probabilidad de error de los canales BSC. \f$ P(U_m \neq U_0| U_0)=\rho \f$.
 *  \param[in] M Es el número de canales BSC.
 *  \param[out] HCond Entropia condicionada. \f$ h(U_0| \rho,M)=H(U_0|U_1,...,U_M)\f$.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: Ps==NULL.
 *  \ingroup PdsITGroup
 */
int pds_symetric_entropy_u0_omega_bsc_model(double rho,PdsBaNatural M, double *HCond);


/** \fn int pds_symetric_joint_entropy_bsc_model(double rho,PdsRaNatural M, double *HJoint)
 *  \brief Encuentra la entropia conjunta. \f$ h( \rho,M)=H(U_1,...,U_M)\f$.
 *  Este es el caso cuando todas las probabilidades de error son iguales a rho. \f$rho=\rho\f$.
 *
 *  Dadas M fontes Um , m ={1,2,...,M}, generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=0.5, atraves de M canales BSC
 *  con probabilidades de error Ps=[rho, rho, ..., rho].
 *
 *  \f[ P_k(\rho,M)= 0.5 \{ \rho^k (1-\rho)^{M-k} + \rho^{M-k} (1-\rho)^{k} \} \f]

 *  \f[ h(\rho,M)= \sum_{k=0}^{M}  {M \choose k} P_k(\rho,M)~log_2 ( P_k(\rho,M)    ) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)=\rho \f]
 *
 *  \param[in] rho Es la probabilidad de error de los canales BSC. \f$ P(U_m \neq U_0| U_0)=\rho \f$.
 *  \param[in] M Es el número de canales BSC.
 *  \param[out] HJoint Entropia conjunta. \f$ h(\rho,M)=H(U_1,...,U_M)\f$.
 *  \return Sempre retorna TRUE.
 *  \ingroup PdsITGroup
 */
int pds_symetric_joint_entropy_bsc_model(double rho,PdsRaNatural M, double *HJoint);


/** \fn int pds_inv_symetric_entropy_u0_omega_bsc_model(double HCond,short int M,double * rho)
 *  \brief Encuentra la inversa de la entropia condicionada. \f$ HCond=h(U_0| \rho,M)=H(U_0|U_1,...,U_M)\f$.
 *  Esta funcion es la inversa de pds_symetric_entropy_u0_omega_bsc_model().
 *  Cuando HCond y M son entregados la funcion retorna el valor de rho.
 *  Este es el caso cuando todas las probabilidades de error son iguales a rho. \f$rho=\rho\f$.

 *
 *  Dadas M fontes Um , m ={1,2,...,M}, generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=0.5, atraves de M canales BSC
 *  con probabilidades de error Ps=[rho, rho, ..., rho].
 *
 *  \f[ h(U_0| \rho,M)= \sum_{k=0}^{M} {M \choose k} \rho^k (1-\rho)^{M-k}~log_2 \left ( 1 + \{ \frac{\rho}{1-\rho} \}^{M-2 k} \right ) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)=\rho \f]
 *
 *  \param[in] HCond Entropia condicionada. \f$ HCond=h(U_0| \rho,M)=H(U_0|U_1,...,U_M)\f$.
 *  \param[in] M Es el número de canales BSC.
 *  \param[out] rho Es la probabilidad de error de los canales BSC. \f$ P(U_m \neq U_0| U_0)=\rho \f$.
 *  \return TRUE si todo fue bien, o FALSE sino. Ejem: rho==NULL.
 *  \ingroup PdsITGroup
 */
int pds_inv_symetric_entropy_u0_omega_bsc_model(double HCond,short int M,double * rho);


/** \fn int pds_inv_symetric_joint_entropy_bsc_model(double HJoint,short int M,double * rho)
 *  \brief Encuentra la inversa de la entropia conjunta. \f$ HJoint=h( \rho,M)=H(U_1,...,U_M)\f$.
 *  Esta funcion es la inversa de pds_symetric_joint_entropy_bsc_model().
 *  Cuando HJoint y M son entregados la funcion retorna el valor de rho.
 *  Este es el caso cuando todas las probabilidades de error son iguales a rho. \f$rho=\rho\f$.
 *
 *  Dadas M fontes Um , m ={1,2,...,M}, generadas
 *  pasando una fuente U0, con probabilidade Pr(U0=1)=0.5, atraves de M canales BSC
 *  con probabilidades de error Ps=[rho, rho, ..., rho].
 *
 *  \f[ P_k(\rho,M)= 0.5 \{ \rho^k (1-\rho)^{M-k} + \rho^{M-k} (1-\rho)^{k} \} \f]
 *
 *  \f[ h(\rho,M)= \sum_{k=0}^{M}  {M \choose k} P_k(\rho,M)~log_2 ( P_k(\rho,M)    ) \f]
 *   
 *  \f[ P(U_m \neq U_0| U_0)=\rho \f]
 *
 *  \param[in] HJoint Entropia conjunta. \f$ h(\rho,M)=H(U_1,...,U_M)\f$.
 *  \param[in] M Es el número de canales BSC.
 *  \param[out] rho Es la probabilidad de error de los canales BSC. \f$ P(U_m \neq U_0| U_0)=\rho \f$.
 *  \return Sempre retorna TRUE.
 *  \ingroup PdsITGroup
 */
int pds_inv_symetric_joint_entropy_bsc_model(double HJoint,short int M,double * rho);
/**
 * @}
 */

#ifdef __cplusplus
}
#endif 


#endif /* __PDSIT_H__ */
